/*-
 * Copyright (C) 2009 Gabor Kovesdan <gabor@FreeBSD.org>
 * 2012 Oleg Moskalenko oleg.moskalenko@citrix.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#if !defined(__BWSTRING_H__)
#define __BWSTRING_H__

#include <stdbool.h>
#include <stdio.h>
#include <errno.h>
#include <sysexits.h>
#include <wchar.h>

#include "mem.h"

extern bool byte_sort;

/* wchar_t is of 4 bytes: */
#define SIZEOF_WCHAR_STRING(LEN) ((LEN)*sizeof(wchar_t))

/*
 * Binary "wide" string
 */
struct _bwstring;
typedef struct _bwstring bwstring;
typedef void* bwstring_iterator;

/*
 * Binary "wide" string
 */
struct _bwstring
{
	size_t len;
	size_t sz;
	union
	{
		wchar_t wstr[1];
		unsigned char cstr[1];
	} data;
};

#define bwslen(s) ((s)->len)

bwstring* bwsalloc(size_t sz);

size_t bwssize(const bwstring *bws);
void bws_setlen(bwstring* bws, size_t newlen);
size_t bws_memsize(const bwstring* bws);
double bwstod(bwstring *s0, bool *empty);
int bws_month_score(const bwstring *s0);

bwstring *ignore_leading_blanks(bwstring *str);
bwstring *ignore_nonprinting(bwstring *str);
bwstring *dictionary_order(bwstring *str);
bwstring *ignore_case(bwstring *str);

void bwsprintf(FILE*, bwstring*,const char* prefix, const char* suffix);
void bws_disorder_warnx(bwstring* s, const char* fn, size_t pos);

bwstring* bwsdup(const bwstring *s);
bwstring* bwssbdup(const wchar_t *str, size_t size);
bwstring* bwscsbdup(const unsigned char *str, size_t size);
void bwsfree(const bwstring* s);
size_t bwscpy(bwstring *dst, const bwstring *src);
bwstring* bwsncpy(bwstring *dst, const bwstring *src, size_t size);
bwstring* bwsnocpy(bwstring *dst, const bwstring *src, size_t offset, size_t size);
int bwscoll(const bwstring* bws1, const bwstring *bws2);
int bwscasecoll(const bwstring* bws1, const bwstring *bws2);
int bwsfwrite(bwstring* bws, FILE* f, bool zero_ended);
bwstring *bwsfgetln(FILE* file, size_t *len, bool zero_ended);

static inline bwstring_iterator bws_begin(bwstring *bws) {
  return (bwstring_iterator)(&(bws->data));
}

static inline bwstring_iterator bws_end(bwstring *bws) {
  if(MB_CUR_MAX == 1) {
    return (bwstring_iterator)(bws->data.cstr+bws->len);
  } else {
    return (bwstring_iterator)(bws->data.wstr+bws->len);
  }
}

static inline bwstring_iterator bws_iterator_inc(bwstring_iterator iter, size_t pos) {
  if(MB_CUR_MAX == 1) {
    return ((unsigned char *)iter)+pos;
  } else {
    return ((wchar_t*)iter)+pos;
  }
}

static inline wchar_t bws_get_iter_value(bwstring_iterator iter) {
  if(MB_CUR_MAX == 1) {
    return *((unsigned char *)iter);
  } else {
    return *((wchar_t*)iter);
  }
}

#define bws_get(bws, pos) ((MB_CUR_MAX == 1) ? ((bws)->data.cstr[(pos)]) : (bws)->data.wstr[(pos)])

#endif /* __BWSTRING_H__ */
