/*-
 * Copyright (C) 2009 Gabor Kovesdan <gabor@FreeBSD.org>
 * 2012 Oleg Moskalenko oleg.moskalenko@citrix.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#if !defined(__COLL_H__)
#define __COLL_H__

#include "sort.h"
#include "bwstring.h"

/*
 * If true, we output some debug information.
 */
extern bool debug_sort;

/*
 * Sort hint data for -n
 */
typedef struct _n_hint
{
	bool neg1;
	unsigned long long n1;
} n_hint;

/*
 * Sort hint data for -g
 */
typedef struct _g_hint
{
	double d;
	bool nan;
	bool notnum;
} g_hint;

/*
 * Sort hint data for -M
 */
typedef struct _M_hint
{
	int m;
} M_hint;

/*
 * Status of a sort hint object
 */
typedef enum
{
	HS_ERROR=-1,
	HS_UNINITIALIZED=0,
	HS_INITIALIZED=1
} hint_status;

/*
 * Sort hint object
 */
typedef struct _key_hint
{
	hint_status status;
	union {
		n_hint nh;
		g_hint gh;
		M_hint Mh;
	} v;
} key_hint;

/*
 * Key value
 */
struct _key_value
{
	bwstring* k; /* key string */
	bool owner; /* flag whether the key owns the string */
	key_hint kh; /* key sort hint */
};

/*
 * Set of keys container object.
 */
typedef struct _keys_set
{
	size_t nums;
	key_value* set;
} keys_set;

/*
 * Parsed -k option data
 */
typedef struct _key_specs
{
	size_t f1;
	size_t c1;
	bool pos1b;
	size_t f2;
	size_t c2;
	bool pos2b;
	sort_mods sm;
} key_specs;

/*
 * Single entry in sort list.
 */
typedef struct _sort_list_item
{
	bwstring* str;
	keys_set* ks;
} sort_list_item;

/*
 * List of data to be sorted.
 */
typedef struct _sort_list
{
	int count;
	int size;
	unsigned long long memsize;
	sort_list_item** list;
} sort_list;

extern key_specs* keys;
extern size_t keys_num;

/*
 * Main localised symbols
 */
extern wchar_t symbol_decimal_point;
extern wchar_t symbol_thousands_sep;
extern wchar_t symbol_negative_sign;
extern wchar_t symbol_positive_sign;

/* funcs */

cmpcoll_t get_sort_func(sort_mods *sm);
keys_set* alloc_keys_set(void);
void free_keys_set(keys_set* ks);
void set_key_on_keys_set(keys_set *ks, bwstring *s, size_t ind, bool owner);

void sort_list_item_init(sort_list_item* si);
size_t sort_list_item_set(sort_list_item* si, bwstring* str);
void sort_list_item_clean(sort_list_item* si);
void sort_list_item_swap(sort_list_item* si1, sort_list_item* si2);

keys_set *preproc(bwstring *s);
int str_coll(const bwstring *, const bwstring *);
int key_coll(const keys_set *ks1, const keys_set *ks2);
int str_list_coll(bwstring *str1, sort_list_item **ss2);
int list_coll_by_str_only(sort_list_item **ss1, sort_list_item **ss2);
int list_coll(sort_list_item **ss1, sort_list_item **ss2);

#endif /* __COLL_H__ */
