/*-
 * Copyright (C) 2009 Gabor Kovesdan <gabor@FreeBSD.org>
 * 2012 Oleg Moskalenko oleg.moskalenko@citrix.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#if !defined(__SORT_FILE_H__)
#define __SORT_FILE_H__

#include "sort.h"
#include "coll.h"

#define		SORT_DEFAULT	0
#define		SORT_QSORT	1
#define		SORT_MERGESORT	2
#define		SORT_HEAPSORT	3
#define		SORT_RADIXSORT  4

/*
 * File reader object 
 */
struct _file_reader;
typedef struct _file_reader file_reader;

/*
 * List of files to be sorted
 */
typedef struct _file_list
{
	int count;
	int sz;
	char** fns;
	bool tmp;
} file_list;

/* memory */

/**/

extern unsigned long long free_memory;
extern unsigned long long available_free_memory;

/* temporary file dir */

extern const char* tmpdir;

/*
 * Max number of simultaneously open files (including the output file).
 */
extern size_t max_open_files;

/* funcs */

file_reader* file_reader_init(const char* fsrc);
bwstring* file_reader_readline(file_reader* fr);
void file_reader_free(file_reader* fr);

char* read_file0_line(FILE* f);

void init_tmp_files(void);
void clear_tmp_files(void);
char* new_tmp_file_name(void);

void file_list_init(file_list *fl, bool tmp);
void file_list_add(file_list* fl, char* fn, bool allocate);
void file_list_populate(file_list* fl, int argc, char** argv, bool allocate);
void file_list_clean(file_list *fl);

int check(const char *);
void merge_files(file_list *fl, const char *fn_out);
FILE *openfile(const char *, const char *);
int procfile(const char* fn, sort_list *list, file_list *fl);

void sort_list_init(sort_list *l);
void sort_list_add(sort_list *l, bwstring* str);
void sort_list_clean(sort_list *l);
void sort_list_dump(sort_list *l, const char* fn);

void sort_list_to_file(sort_list *list, const char* outfile);

#endif /* __SORT_FILE_H__ */
